@@ -99,10 +99,12 @@
 #include "kernel-shared/disk-io.h"
 #include "kernel-shared/volumes.h"
 #include "kernel-shared/transaction.h"
-#include "crypto/crc32c.h"
+#include "kernel-shared/file-item.h"
+#include "crypto/hash.h"
 #include "common/defs.h"
 #include "common/extent-cache.h"
 #include "common/internal.h"
+#include "common/cpu-utils.h"
 #include "common/messages.h"
 #include "common/task-utils.h"
 #include "common/path-utils.h"
@@ -112,6 +114,7 @@
 #include "common/device-scan.h"
 #include "common/box.h"
 #include "common/open-utils.h"
+#include "cmds/commands.h"
 #include "check/repair.h"
 #include "mkfs/common.h"
 #include "convert/common.h"
@@ -179,7 +182,8 @@ static int csum_disk_extent(struct btrfs_trans_handle *trans,
 			    struct btrfs_root *root,
 			    u64 disk_bytenr, u64 num_bytes)
 {
-	u32 blocksize = root->fs_info->sectorsize;
+	struct btrfs_fs_info *fs_info = trans->fs_info;
+	u32 blocksize = fs_info->sectorsize;
 	u64 offset;
 	char *buffer;
 	int ret = 0;
@@ -188,14 +192,21 @@ static int csum_disk_extent(struct btrfs_trans_handle *trans,
 	if (!buffer)
 		return -ENOMEM;
 	for (offset = 0; offset < num_bytes; offset += blocksize) {
-		ret = read_disk_extent(root, disk_bytenr + offset,
-					blocksize, buffer);
+		u64 read_len = blocksize;
+
+		ret = read_data_from_disk(fs_info, buffer,
+					  disk_bytenr + offset, &read_len, 0);
 		if (ret)
 			break;
-		ret = btrfs_csum_file_block(trans,
-					    disk_bytenr + num_bytes,
-					    disk_bytenr + offset,
-					    buffer, blocksize);
+		if (read_len == 0) {
+			error("failed to read logical bytenr %llu",
+			      disk_bytenr + offset);
+			ret = -EIO;
+			break;
+		}
+		ret = btrfs_csum_file_block(trans, disk_bytenr + offset,
+					    BTRFS_EXTENT_CSUM_OBJECTID,
+					    fs_info->csum_type, buffer);
 		if (ret)
 			break;
 	}
@@ -356,7 +367,6 @@ static int migrate_one_reserved_range(struct btrfs_trans_handle *trans,
 	u64 hole_len;
 	struct cache_extent *cache;
 	struct btrfs_key key;
-	struct extent_buffer *eb;
 	int ret = 0;
 
 	/*
@@ -367,6 +377,8 @@ static int migrate_one_reserved_range(struct btrfs_trans_handle *trans,
 	 * migrate ranges that covered by old fs data.
 	 */
 	while (cur_off < range_end(range)) {
+		void *buf;
+
 		cache = search_cache_extent(used, cur_off);
 		if (!cache)
 			break;
@@ -388,25 +400,20 @@ static int migrate_one_reserved_range(struct btrfs_trans_handle *trans,
 		if (ret < 0)
 			break;
 
-		eb = malloc(sizeof(*eb) + cur_len);
-		if (!eb) {
+		buf = malloc(cur_len);
+		if (!buf) {
 			ret = -ENOMEM;
 			break;
 		}
 
-		ret = pread(fd, eb->data, cur_len, cur_off);
+		ret = pread(fd, buf, cur_len, cur_off);
 		if (ret < cur_len) {
 			ret = (ret < 0 ? ret : -EIO);
-			free(eb);
+			free(buf);
 			break;
 		}
-		eb->start = key.objectid;
-		eb->len = key.offset;
-		eb->fs_info = root->fs_info;
-
-		/* Write the data */
-		ret = write_and_map_eb(root->fs_info, eb);
-		free(eb);
+		ret = write_data_to_disk(root->fs_info, buf, key.objectid, key.offset);
+		free(buf);
 		if (ret < 0)
 			break;
 
@@ -1165,7 +1172,7 @@ static int do_convert(const char *devname, u32 convert_flags, u32 nodesize,
 	if (ret)
 		goto fail;
 
-	ASSERT(cctx.total_bytes != 0);
+	UASSERT(cctx.total_bytes != 0);
 	blocksize = cctx.blocksize;
 	if (blocksize < 4096) {
 		error("block size is too small: %u < 4096", blocksize);
@@ -1792,28 +1799,44 @@ free_mem:
 	return ret;
 }
 
-static void print_usage(void)
-{
-	printf("usage: btrfs-convert [options] device\n");
-	printf("options:\n");
-	printf("\t-d|--no-datasum        disable data checksum, sets NODATASUM\n");
-	printf("\t-i|--no-xattr          ignore xattrs and ACLs\n");
-	printf("\t-n|--no-inline         disable inlining of small files to metadata\n");
-	printf("\t--csum TYPE\n");
-	printf("\t--checksum TYPE        checksum algorithm to use (default: crc32c)\n");
-	printf("\t-N|--nodesize SIZE     set filesystem metadata nodesize\n");
-	printf("\t-r|--rollback          roll back to the original filesystem\n");
-	printf("\t-l|--label LABEL       set filesystem label\n");
-	printf("\t-L|--copy-label        use label from converted filesystem\n");
-	printf("\t--uuid SPEC            new, copy or user-defined conforming UUID\n");
-	printf("\t-p|--progress          show converting progress (default)\n");
-	printf("\t-O|--features LIST     comma separated list of filesystem features\n");
-	printf("\t--no-progress          show only overview, not the detailed progress\n");
-	printf("\n");
-	printf("Supported filesystems:\n");
-	printf("\text2/3/4: %s\n", BTRFSCONVERT_EXT2 ? "yes" : "no");
-	printf("\treiserfs: %s\n", BTRFSCONVERT_REISERFS ? "yes" : "no");
-}
+static const char * const convert_usage[] = {
+	"btrfs-convert [options] device",
+	"In-place conversion from other filesystems to BTRFS",
+	"",
+	"Options:",
+	OPTLINE("-d|--no-datasum", "disable data checksum, sets NODATASUM"),
+	OPTLINE("-i|--no-xattr", "ignore xattrs and ACLs"),
+	OPTLINE("-n|--no-inline", "disable inlining of small files to metadata"),
+	OPTLINE("--csum TYPE", ""),
+	OPTLINE("--checksum TYPE", "checksum algorithm to use (default: crc32c)"),
+	OPTLINE("-N|--nodesize SIZE", "set filesystem metadata nodesize"),
+	OPTLINE("-r|--rollback", "roll back to the original filesystem"),
+	OPTLINE("-l|--label LABEL", "set filesystem label"),
+	OPTLINE("-L|--copy-label", "use label from converted filesystem"),
+	OPTLINE("--uuid SPEC", "new, copy or user-defined conforming UUID"),
+	OPTLINE("-p|--progress", "show converting progress (default)"),
+	OPTLINE("-O|--features LIST", "comma separated list of filesystem features"),
+	OPTLINE("--no-progress", "show only overview, not the detailed progress"),
+	"",
+	"Supported filesystems:",
+	"\text2/3/4: "
+#if BTRFSCONVERT_EXT2
+	"yes",
+#else
+	"no",
+#endif
+	"\treiserfs: "
+#if BTRFSCONVERT_REISERFS
+	"yes",
+#else
+	"no",
+#endif
+	NULL
+};
+
+static const struct cmd_struct convert_cmd = {
+	.usagestr = convert_usage
+};
 
 int BOX_MAIN(convert)(int argc, char *argv[])
 {
@@ -1834,7 +1857,8 @@ int BOX_MAIN(convert)(int argc, char *argv[])
 	u32 copy_fsid = 0;
 	char fsid[BTRFS_UUID_UNPARSED_SIZE] = {0};
 
-	crc32c_optimization_init();
+	cpu_detect_flags();
+	hash_init_accel();
 	btrfs_assert_feature_buf_size();
 	printf("btrfs-convert from %s\n\n", PACKAGE_STRING);
 
@@ -1952,13 +1976,12 @@ int BOX_MAIN(convert)(int argc, char *argv[])
 				break;
 			case GETOPT_VAL_HELP:
 			default:
-				print_usage();
-				return c != GETOPT_VAL_HELP;
+				usage(&convert_cmd, c != GETOPT_VAL_HELP);
 		}
 	}
 	set_argv0(argv);
 	if (check_argc_exact(argc - optind, 1)) {
-		print_usage();
+		usage(&convert_cmd, 1);
 		return 1;
 	}
 
@@ -1969,7 +1992,7 @@ int BOX_MAIN(convert)(int argc, char *argv[])
 	}
 
 	if (usage_error) {
-		print_usage();
+		usage(&convert_cmd, 1);
 		return 1;
 	}
 
